#!/bin/sh

#  FLAC - Free Lossless Audio Codec
#  Copyright (C) 2001,2002,2003,2004,2005,2006  Josh Coalson
#
#  This file is part the FLAC project.  FLAC is comprised of several
#  components distributed under difference licenses.  The codec libraries
#  are distributed under Xiph.Org's BSD-like license (see the file
#  COPYING.Xiph in this distribution).  All other programs, libraries, and
#  plugins are distributed under the GPL (see COPYING.GPL).  The documentation
#  is distributed under the Gnu FDL (see COPYING.FDL).  Each file in the
#  FLAC distribution contains at the top the terms under which it may be
#  distributed.
#
#  Since this particular file is relevant to all components of FLAC,
#  it may be distributed under the Xiph.Org license, which is the least
#  restrictive of those mentioned above.  See the file COPYING.Xiph in this
#  distribution.

die ()
{
	echo $* 1>&2
	exit 1
}

if [ x = x"$1" ] ; then 
	BUILD=debug
else
	BUILD="$1"
fi

PATH=../../build/bin:$PATH
BINS_PATH=../test_files/bins

if [ x"$FLAC__TEST_LEVEL" = x ] ; then
	FLAC__TEST_LEVEL=1
fi

flake -h 1>/dev/null 2>/dev/null || die "ERROR can't find flake executable"

run_flac ()
{
	if [ x"$FLAC__TEST_WITH_VALGRIND" = xyes ] ; then
		echo "valgrind --leak-check=yes --show-reachable=yes --num-callers=100 flake $*" >>test_bins.valgrind.log
		valgrind --leak-check=yes --show-reachable=yes --num-callers=100 --logfile-fd=4 flake $* 4>>test_bins.valgrind.log
	else
		flake $*
	fi
}

test -d ${BINS_PATH} || exit 77

test_file ()
{
	name=$1
	channels=$2
	bps=$3
	encode_options="$4"

	echo -n "$name.bin (--channels=$channels --bps=$bps $encode_options): encode..."
	cmd="run_flac $encode_options z.wav"
	echo "### ENCODE $name #######################################################" >> ./streams.log
	echo "###    cmd=$cmd" >> ./streams.log
	$cmd 2>>./streams.log || die "ERROR during encode of $name"

	echo -n "decode..."
	cmd="flac --silent --force --endian=big --sign=signed --decode --force-raw-format z.flac -o z.raw";
	echo "### DECODE $name #######################################################" >> ./streams.log
	echo "###    cmd=$cmd" >> ./streams.log
	$cmd 2>>./streams.log || die "ERROR during decode of $name"

	ls -1l $name.bin >> ./streams.log
	ls -1l z.flac >> ./streams.log
	ls -1l z.raw >> ./streams.log

	echo -n "compare..."
	cmp $name.bin z.raw || die "ERROR during compare of $name"

	echo OK
}

echo "Testing bins..."
for f in b00 b01 b02 b03 b04 ; do
	binfile=$BINS_PATH/$f
	if [ -f $binfile.bin ] ; then
		#@@@for channels in 1 2 4 8 ; do
		for channels in 1 2 4 ; do
			#@@@for bps in 8 16 24 ; do
			for bps in 16 ; do
				# convert to wav
				flac -V0f --silent --force-raw-format --endian=big --sign=signed --sample-rate=44100 --bps=$bps --channels=$channels --no-padding --no-seektable $binfile.bin -c | flac --silent -df -o z.wav - || die "ERROR converting to wav"
				for extras in '' '-v 1' ; do
					for opt in 0 1 2 3 4 5 6 7 8 9 10 11 12 ; do
						test_file $binfile $channels $bps "-$opt $extras"
					done
				done
				test_file $binfile $channels $bps "-t 0"
				test_file $binfile $channels $bps "-t 1 -l 0,4"
				for ol in 1 8 12 32 ; do
					for om in 0 1 2 3 4 5 6 ; do
						for os in 0 1 ; do
							if [ $os -eq 0 ] || [ $channels -eq 2 ] ; then
								for or in 0 4 8 ; do
									for blocksize in '-b 32' '-b 1152' '-b 4608' '-b 65535' ; do
										test_file $binfile $channels $bps "-3 -l 1,$ol -m $om -s $os -r 0,$or $blocksize"
									done
								done
							fi
						done
					done
				done
			done
		done
	else
		echo "$binfile not found, skipping."
	fi
done
